#!/usr/bin/python
# -*- coding: utf-8 -*-


import unittest
from maltiverse import Maltiverse
import time


class TestMaltiverse(unittest.TestCase):
    """Test for Maltiverse class"""

    def __init__(self, *args, **kwargs):
        super(TestMaltiverse, self).__init__(*args, **kwargs)
        self.email = "foo@mail.com"
        self.password = "fakepasswd"

    def test_login(self):
        """Test that performs an login"""
        m = Maltiverse()
        response_login = m.login(email=self.email, password=self.password)
        self.assertTrue(response_login, True)

    def test_ip_get(self):
        """Test that performs an ip lookup"""
        m = Maltiverse()
        response_login = m.login(email=self.email, password=self.password)
        self.assertTrue(response_login, True)
        item = m.ip_get("1.1.1.1")
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("ip_addr" in item)

    def test_hostname_get(self):
        """Test that performs a hostname lookup"""
        m = Maltiverse()
        response_login = m.login(email=self.email, password=self.password)
        self.assertTrue(response_login, True)
        item = m.hostname_get("amazon.es")
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("hostname" in item)

    def test_search(self):
        """Test that performs search into the platform"""
        m = Maltiverse()
        print(m.login(email=self.email, password=self.password))
        item = m.search('country_code:"CN"', fr=0, size=2)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))

    def test_ip_put_delete(self):
        """Test that performs a IP put and a delete"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        ip_dict = {
            "blacklist": [{"description": "test", "source": "test"}],
            "classification": "whitelisted",
            "ip_addr": "60.60.60.60",
            "type": "ip",
        }
        item = m.ip_put(ip_dict)
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(
            item["message"] == "IP created" or item["message"] == "IP updated"
        )

        time.sleep(5)

        item = m.ip_delete(ip_dict["ip_addr"])
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(item["message"] == "IP deleted")

    def test_hostname_put_delete(self):
        """Test that performs a hostname put and a delete"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        hostname_dict = {
            "blacklist": [{"description": "test", "source": "test"}],
            "classification": "malicious",
            "domain": "testtesttesttesttesttesttest.com",
            "hostname": "testtesttesttesttesttesttest.com",
            "tld": "com",
            "type": "hostname",
        }
        item = m.hostname_put(hostname_dict)
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(
            item["message"] == "Hostname created"
            or item["message"] == "Hostname updated"
        )

        # Wait for five seconds to permit DB transaction happen
        time.sleep(5)

        item = m.hostname_delete(hostname_dict["hostname"])
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(item["message"] == "Hostname deleted")

    def test_url_get(self):
        """Test that performs an url lookup"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        item = m.url_get("http://m.tpsservices.runescape.com-no.ru/")
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("url" in item)

    def test_url_put_delete(self):
        """Test that performs a url put and a delete"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        url_dict = {
            "blacklist": [{"description": "test", "source": "test"}],
            "domain": "test.com",
            "hostname": "www.test.com",
            "type": "url",
            "url": "http://www.test.com/test.php",
        }
        item = m.url_put(url_dict)
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(
            item["message"] == "Url created" or item["message"] == "Url updated"
        )

        # Wait for two seconds to permit DB transaction happen
        time.sleep(5)

        item = m.url_delete(url_dict["url"])
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(item["message"] == "Url deleted")

    def test_sample_get(self):
        """Test that performs an sample lookup"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        item = m.sample_get(
            "3b9d4f379e59cfc5ed8217424c833fbd16e7bff322c2ea696870061bbd2c5273"
        )
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("sha256" in item)

    def test_sample_put_delete(self):
        """Test that performs a sample put and a delete"""
        m = Maltiverse()
        m.login(email=self.email, password=self.password)
        sample_dict = {
            "blacklist": [{"description": "test", "source": "test"}],
            "classification": "whitelisted",
            "filename": ["test"],
            "md5": "00000000000000000000000000000000",
            "sha1": "0000000000000000000000000000000000000000",
            "sha256": "0000000000000000000000000000000000000000000000000000000000000000",
            "sha512": "00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000",
            "type": "sample",
        }
        item = m.sample_put(sample_dict)
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(
            item["message"] == "Sample created" or item["message"] == "Sample updated"
        )

        # Wait for two seconds to permit DB transaction happen
        time.sleep(5)

        item = m.sample_delete(sample_dict["sha256"])
        print(item)
        self.assertTrue(isinstance(str(item), str))
        self.assertTrue(isinstance(item, dict))
        self.assertTrue("status" in item)
        self.assertTrue(item["status"] == "success")
        self.assertTrue(item["message"] == "Sample deleted")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import hashlib
import json
import typing as t

import jwt
import requests

T_AdminIndexScope = t.Literal["open", "restricted", "sandbox"]


class Maltiverse:
    def __init__(self, auth_token=None, endpoint="https://api.maltiverse.com"):
        self.endpoint = endpoint
        self.auth_token = auth_token
        self.sub = None
        self.team_name = None
        self.team_researcher = None
        self.admin = None
        self._default_headers = self._create_headers()
        if self.auth_token:
            self._decode_token()

    def _create_headers(self):
        """Create default headers with or without authentication token."""
        headers = {"Accept": "application/json"}
        if self.auth_token:
            headers["Authorization"] = f"Bearer {self.auth_token}"
        return headers

    def _update_headers(self, additional_headers):
        """Update headers with additional headers provided."""
        return {**self._default_headers, **additional_headers}

    def prepare_put_payload(self, payload: dict):
        """Prepare the payload for PUT requests, removing fields based on user permissions."""
        if self.team_researcher and not self.admin and "blacklist" in payload:
            self._sanitize_blacklist(payload)
        return json.dumps(payload)

    def _prepare_put_params(self, index_scope: t.Optional[T_AdminIndexScope] = None):
        """Prepare query params for PUT requests (index_scope is query-level)."""
        if self.admin and index_scope is not None:
            return {"index_scope": index_scope}
        return {}

    def _sanitize_blacklist(self, params):
        """Clean restricted fields and add required information to the blacklist."""
        restricted_fields = [
            "creation_time",
            "modification_time",
            "domain",
            "urlchecksum",
            "tld",
            "type",
        ]
        for field in restricted_fields:
            params.pop(field, None)

        for bl_item in params.get("blacklist", []):
            bl_item["ref"] = self.sub
            bl_item["source"] = self.team_name
            bl_item.pop("first_seen", None)
            bl_item.pop("last_seen", None)

    def login(self, email, password):
        """Logs in and stores the authentication token and user details."""
        response = requests.post(
            f"{self.endpoint}/auth/login",
            data=json.dumps({"email": email, "password": password}),
            headers=self._update_headers({"Content-Type": "application/json"}),
        ).json()

        if response.get("status") == "success" and "auth_token" in response:
            self.auth_token = response["auth_token"]
            self._default_headers = self._create_headers()
            self._decode_token()
            return True
        raise Exception("Login Failed")

    def _decode_token(self):
        """Decodes the JWT token and stores user details."""
        decoded_payload = jwt.decode(
            self.auth_token, options={"verify_signature": False}
        )
        self.sub = decoded_payload.get("sub")
        self.team_name = decoded_payload.get("team_name")
        self.team_researcher = decoded_payload.get("team_researcher")
        self.admin = decoded_payload.get("admin")

    def _request(self, method, url, headers=None, **kwargs):
        """Make an HTTP request with the specified method."""
        headers = headers or self._default_headers
        return requests.request(method, url, headers=headers, **kwargs).json()

    def ip_get(self, ip_addr):
        """Fetch information for a given IP address."""
        return self._request("GET", f"{self.endpoint}/ip/{ip_addr}")

    def ip_put(self, ip_dict: dict, index_scope: t.Optional[T_AdminIndexScope] = None):
        """Update or insert an IP address observable."""
        return self._request(
            "PUT",
            f"{self.endpoint}/ip/{ip_dict['ip_addr']}",
            headers=self._update_headers({"Content-Type": "application/json"}),
            params=self._prepare_put_params(index_scope=index_scope),
            data=self.prepare_put_payload(ip_dict),
        )

    def ip_delete(self, ip_addr):
        """Delete an IP address observable."""
        return self._request("DELETE", f"{self.endpoint}/ip/{ip_addr}")

    def hostname_get(self, hostname):
        """Fetch information for a given hostname."""
        return self._request("GET", f"{self.endpoint}/hostname/{hostname}")

    def hostname_put(
        self, hostname_dict: dict, index_scope: t.Optional[T_AdminIndexScope] = None
    ):
        """Update or insert a hostname observable."""
        return self._request(
            "PUT",
            f"{self.endpoint}/hostname/{hostname_dict['hostname']}",
            headers=self._update_headers({"Content-Type": "application/json"}),
            params=self._prepare_put_params(index_scope=index_scope),
            data=self.prepare_put_payload(hostname_dict),
        )

    def hostname_delete(self, hostname):
        """Delete a hostname observable."""
        return self._request("DELETE", f"{self.endpoint}/hostname/{hostname}")

    def url_get(self, url):
        """Fetch information for a given URL."""
        urlchecksum = hashlib.sha256(url.encode("utf-8")).hexdigest()
        return self._request("GET", f"{self.endpoint}/url/{urlchecksum}")

    def url_get_by_checksum(self, urlchecksum):
        """Fetch a URL by its SHA256 checksum."""
        return self._request("GET", f"{self.endpoint}/url/{urlchecksum}")

    def url_put(
        self, url_dict: dict, index_scope: t.Optional[T_AdminIndexScope] = None
    ):
        """Update or insert a URL observable."""
        urlchecksum = hashlib.sha256(url_dict["url"].encode("utf-8")).hexdigest()
        return self._request(
            "PUT",
            f"{self.endpoint}/url/{urlchecksum}",
            headers=self._update_headers({"Content-Type": "application/json"}),
            params=self._prepare_put_params(index_scope=index_scope),
            data=self.prepare_put_payload(url_dict),
        )

    def url_delete(self, url):
        """Delete a URL observable."""
        urlchecksum = hashlib.sha256(url.encode("utf-8")).hexdigest()
        return self._request("DELETE", f"{self.endpoint}/url/{urlchecksum}")

    def sample_get(self, sample, algorithm="sha256"):
        """Fetch a sample using a hash algorithm (MD5, SHA1, SHA256, SHA512)."""
        hash_methods = {
            "md5": self.sample_get_by_md5,
            "sha1": self.sample_get_by_sha1,
            "sha256": self.sample_get_by_sha256,
            "sha512": self.sample_get_by_sha512,
        }
        return hash_methods.get(algorithm, self.sample_get_by_sha256)(sample)

    def sample_get_by_md5(self, md5):
        return self._request("GET", f"{self.endpoint}/sample/md5/{md5}")

    def sample_get_by_sha1(self, sha1):
        return self._request("GET", f"{self.endpoint}/sample/sha1/{sha1}")

    def sample_get_by_sha256(self, sha256):
        return self._request("GET", f"{self.endpoint}/sample/{sha256}")

    def sample_get_by_sha512(self, sha512):
        return self._request("GET", f"{self.endpoint}/sample/sha512/{sha512}")

    def sample_put(
        self, sample_dict: dict, index_scope: t.Optional[T_AdminIndexScope] = None
    ):
        """Update or insert a sample observable."""
        return self._request(
            "PUT",
            f"{self.endpoint}/sample/{sample_dict['sha256']}",
            headers=self._update_headers({"Content-Type": "application/json"}),
            params=self._prepare_put_params(index_scope=index_scope),
            data=self.prepare_put_payload(sample_dict),
        )

    def sample_delete(self, sha256):
        """Delete a sample observable."""
        return self._request("DELETE", f"{self.endpoint}/sample/{sha256}")

    def email_get(self, email_address):
        """Fetch information for a given email address."""
        return self._request("GET", f"{self.endpoint}/email/{email_address}")

    def email_put(
        self, email_dict: dict, index_scope: t.Optional[T_AdminIndexScope] = None
    ):
        """Update or insert an email address observable."""
        return self._request(
            "PUT",
            f"{self.endpoint}/email/{email_dict['email_address']}",
            headers=self._update_headers({"Content-Type": "application/json"}),
            params=self._prepare_put_params(index_scope=index_scope),
            data=self.prepare_put_payload(email_dict),
        )

    def email_delete(self, email_address):
        """Delete an IP address observable."""
        return self._request("DELETE", f"{self.endpoint}/email/{email_address}")

    def search(
        self,
        query,
        fr=None,
        size=None,
        sort=None,
        range=None,
        range_field=None,
        format=None,
    ):
        """Perform a search on the Maltiverse platform."""
        params = {
            "query": query,
            "from": fr,
            "size": size,
            "sort": sort,
            "range": range,
            "range_field": range_field,
            "format": format,
        }
        params = {k: v for k, v in params.items() if v is not None}
        return self._request("GET", f"{self.endpoint}/search", params=params)



from distutils.core import setup

setup(
    name="maltiverse",
    packages=["maltiverse"],
    version="1.2.7",
    license="MIT",
    description="Python API wrapper for Maltiverse",
    author="Antonio Gomez",
    author_email="agm@maltiverse.com",
    url="https://github.com/maltiverse/maltiverse-python",
    download_url="https://github.com/maltiverse/python-maltiverse/archive/master.zip",
    keywords=[
        "maltiverse",
        "API",
        "threat intelligence",
        "IoC",
        "blacklist",
        "search engine",
    ],
    install_requires=["requests", "PyJWT"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)
